# Copyright 2008, Didier Bur,  tweaked by takesh h 2009, modified by bentleykf 2011
#
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted, provided that the above
# copyright notice appear in all copies.
# 
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#----------------------------------------------------------------------------
# Name : Fast texture Writer
# Description : a tool to quickly extract textures from a SU model, to the current directory
# Menu Item : [Plugin] - [Export All Textures]  An entry to menu "Plugin" added by takesh h
# Check sketchucation for changelog and plugin description
# Date : Feb. 12 2008 (tweak by takesh h: April. 17 2009) (mods by bentleykf(niall.campbell@gmail.com): January. 8-15 2011)
# Type : for developers only
# Version: 0.41
# _4 TIG 20141021 compatible with v2014
#----------------------------------------------------------------------------
require 'sketchup.rb'
#Sketchup.send_action "showRubyPanel:"

#	    UI.messagebox "No faces found in selection", MB_MULTILINE
#	    Sketchup::set_status_text("Operation Aborted",SB_PROMPT)

module FastTW

class FastTextureWriter

	@@material_assoc = Hash.new()
	@@checked_components = Hash.new()
	@@tw = false
	
	### correct_filenames added by bentleykf(niall.campbell@gmail.com)
	# Goes through all textures in material library and gives each a
	# unique name for each material. If name conflicts exist a duplicate
	# texture file is written with a different name in the same directory
	# as its sibling
	def self.correct_filenames(msg)
		checked_filenames = Hash.new()
		tw = Sketchup.create_texture_writer
		grp = Sketchup.active_model.entities.add_group

		tick = 0
		count = Sketchup.active_model.materials.length
		Sketchup.active_model.materials.each do |mat|
			tick+=1
			msg_status=msg+": Correcting Texture File Path "+tick.to_s+" of "+count.to_s
			Sketchup::set_status_text(msg_status,SB_PROMPT)
			
			if mat.texture
				filebase = File.basename(mat.texture.filename)
				fileext  = File.extname(mat.texture.filename)
				filename = filebase[0,filebase.length-fileext.length]
				if !checked_filenames[filename]
					checked_filenames[filename] = filebase
					next
				end #if

				#now make this filename unique
				illegal_char = Sketchup.is_valid_filename?(filename)
				filename = filename.delete('^A-Za-z0-9') if(illegal_char)
				if(checked_filenames[filename])
					i=0
					not_unique = true
					while not_unique
						i+=1
						if(!checked_filenames[filename+(i.to_s)])
							filename = filename + (i.to_s)
							not_unique = false
						end #if
					end #while
				end #if
				
				checked_filenames[filename] = filebase
				grp.material = mat
				tw.load(grp)
				filenbase = File.dirname(mat.texture.filename) +"/"+filename+fileext
				File.delete(filenbase) if(File.exists?(filenbase))
				result = tw.write(grp, filenbase)
				
				theight = mat.texture.height
				twidth  = mat.texture.width			
				tex_alpha = mat.alpha
				tex_color = mat.color
				mat.texture = filenbase
				mat.texture.size = [twidth,theight]		
				mat.alpha = tex_alpha
				mat.color = tex_color
				
			end #if
			
		end #each
		grp.erase! if grp.valid?
	end #def

	def self.fast_tw
	
    model=Sketchup.active_model
		#run Jim's sketchup version check
		if Sketchup.version.to_f < 7.0
			model.start_operation("Export All Textures",true)
		else
			model.start_operation("Export All Textures")
		end #if
		
		start = Time.now
		# Create virtual group
		tw = Sketchup.create_texture_writer
		grp = Sketchup.active_model.entities.add_group
		checked_filenames = Hash.new()
		
		fdirname = File.dirname(Sketchup.active_model.path) + "/textures/"
		Dir.mkdir(fdirname, 755) if(!File.directory?(fdirname))

		count = 0
		Sketchup.active_model.materials.each do |mat|
			count+=1 if mat.texture
		end #each
		
		tick = 1
		check = 0
		
		# Iterate through materials, assign to face, load in tw
		Sketchup.active_model.materials.each do |mat|
			if mat.texture
				msg_status="Exporting Texture "+tick.to_s+" of "+count.to_s
				Sketchup::set_status_text(msg_status,SB_PROMPT)
				tick+=1
				texture = mat.texture
				
				
					# --- START --------------  check for filename conflicts
					filebase = File.basename(texture.filename)
					fileext  = File.extname(texture.filename)
					filename = filebase[0,filebase.length-fileext.length]
					
					filename = filename.delete('^A-Za-z0-9') if(Sketchup.is_valid_filename?(filename))
					
					if(checked_filenames[filename])
						i=0
						not_unique = true
						while not_unique
							i+=1
							if(!checked_filenames[filename+(i.to_s)])
								filename = filename + (i.to_s)
								not_unique = false
							end #if
						end #while
					end #if
					# --- END ---------------- end check for filename conflicts
					checked_filenames[filename] = filename #store filename to check conflicts
					
					grp.material = mat
					tw.load(grp)
					# should write texture here????
					#File.delete(fdirname+filename+fileext) if(File.exists?(fdirname+filename+fileext))
					new_filename = fdirname+filename+fileext
					result = tw.write(grp, new_filename)
					
					case result
						when 0
							#puts "file written: "+new_filename+"("+((Time.now-start).to_s)+"s)"
							check+=1
						when 1; puts "file failed (invalid tiff): "+texture.filename
						when 2; puts "file failed (unknown): "+texture.filename
					end #case
				if(texture.filename != new_filename)
					# --- START -------------- update material with path of written texture
						if Sketchup.read_default("Export All Textures","UTP",true)
							tex_size = [texture.width,texture.height]
							tex_alpha = mat.alpha
							tex_color = mat.color
							mat.texture = new_filename
							mat.texture.size = tex_size
							mat.alpha = tex_alpha
							mat.color = tex_color
						end #if
					# --- END ---------------- end update material path
				end #if

			end #if mat.texture
		end #each
		grp.erase! if grp.valid?
		if (check == count)
			finish_time = Time.now-start
			msg_status="Operation Completed: Exported "+check.to_s+" of "+count.to_s+" textures ("+(finish_time.to_s)+"s)"
			puts msg_status
			Sketchup::set_status_text(msg_status,SB_PROMPT)		
			model.commit_operation
		else
			msg_status="Operation Failed: Exported "+check.to_s+" of "+count.to_s+" textures. Run Operation again with ruby console open to see error report."
			UI.messagebox msg_status, MB_OK	
			puts msg_status
			model.abort_operation
		end #if
	end	#def
	
	### added by bentleykf(niall.campbell@gmail.com)
	# adapted from http://boonedocks.net/mike/archives/162-Determining-Image-File-Types-in-Ruby.html
	# checks for jpg,png,tif,tga & bmp and variations in their data formats
	def self.image_type(file)
	
		filestart = IO.read(file,30)
		case filestart
			when /^BM/
				#puts "BMP: "+file
				return 'bmp'
			when /^MM\\x00\*(\\x00){3}/
			  #when /^MM.\*.{3}/
				#puts "TIF MAC: "+file
				return 'tif'
			when /^II\*\\x00\\x08(\\x00){3}\\x15/
			  #when /^II\*./
				#puts "TIF PC: "+file
				return 'tif'
			when /^\\x89PNG/
				#puts "PNG: "+file
				return 'png'
			when /^\\xff\\xd8\\xff\\xe0\\x00\\x10JFIF/
				#puts "JPG: "+file
				return 'jpg'
			when /^\\xff\\xd8\\xff\\xe0/
				#puts "JPG: "+file
				return 'jpg'
			when /^\\xff\\xd8\\xff\\xe1(.*){2}Exif/
				#puts "JPG: "+file
				return 'jpg'
			when /^\\x00\\x00\\x0A\\x00/
				#puts "TGA Compressed: "+file
				return 'tga'
			when /^\\x00\\x00\\x02\\x00/
				#puts "TGA: "+file
				return 'tga'
			else
				#check for tga files by extension
				fileext = File.extname(file)
				case fileext
					when /^\.(?i)(tga|vda|icb|vst)/
						#puts "TGA by extension: "+file
						return 'tga'
					when /^\.(?i)(tif|tiff)/
						#puts "TIF by extension: "+file
						return 'tif'
				end #case
				puts "unknown: "+IO.read(file,20).dump
				return 'unknown'
		end
	end
	
	def self.check_validity(fdirname)
		entries_hash = Hash.new()
		Dir.foreach(fdirname) { |filebase|
			filetype = File.ftype(filebase) #grabs the file type: 'file','directory','characterSpecial','blockSpecial',
																			#'fifo','link','socket','unknown'
			case filetype
				when "file"
					if (!File.readable?(filebase))
						puts "file is not readable: "+filebase
						next
					end #if
					if (!File.size?(filebase))
						puts "file exists but has 0 size: "+filebase
						next
					end #if
					image_type = FastTW::FastTextureWriter.image_type(filebase)
					if(image_type == 'unknown')
						puts "file exists but is unknown: "+filebase
						next
					end #if
					fileext  = File.extname(filebase)
					filename = filebase[0,filebase.length-fileext.length]
					entries_hash[filename] = filename	+ "." + image_type
				when "directory"
					#file is directory, could open?
					#puts "is directory, do nothing"
			end #case
		}
		if(entries_hash.length > 0)
			return entries_hash
		else
			return false
		end #if
	end #def
	
	### fast_tr added by bentleykf(niall.campbell@gmail.com)
	# imports all files in selected folder
	def self.fast_tr
	
    model=Sketchup.active_model
		#run Jim's sketchup version check
		if Sketchup.version.to_f < 7.0
			model.start_operation("Import All Textures",true)
		else
			model.start_operation("Import All Textures")
		end #if
		
		start = Time.now
		dirpath = UI.openpanel("Choose A File Inside The Texture Import Folder", File.dirname(Sketchup.active_model.path), "*.*")
		if (dirpath)
			fdirname = File.dirname(dirpath)
			
			entries_hash = FastTW::FastTextureWriter.check_validity(fdirname)
			if (entries_hash && entries_hash.length > 0)
				tick = 0
				used_textures = Hash.new()
				updated_materials = Hash.new()
				
				#overwrite existing textures if udate existing materials
				#or create surplus materials selected
				if(Sketchup.read_default("Export All Textures","UME",true) || Sketchup.read_default("Export All Textures","UMU",false))
					count = Sketchup.active_model.materials.length
					Sketchup.active_model.materials.each do |mat|
						next if !mat.texture
						texture = mat.texture
						filebase = File.basename(texture.filename)
						fileext  = File.extname(texture.filename)
						filename = filebase[0,filebase.length-fileext.length]
						next if !entries_hash[filename]
						tick +=1
						msg_status="Updating Texture "+tick.to_s+" of "+count.to_s
						Sketchup::set_status_text(msg_status,SB_PROMPT)
						puts msg_status+": "+entries_hash[filename]
						tex_size = [texture.width,texture.height]
						tex_alpha = mat.alpha
						tex_color = mat.color
						mat.texture = fdirname + "/" + entries_hash[filename]
						mat.texture.size = tex_size
						mat.alpha = tex_alpha
						mat.color = tex_color
						used_textures[filename] = entries_hash[filename]
						updated_materials[mat.name] = mat.name
					end #do
				end #if
				
				#import surplus textures if create surplus materials or
				#create new materials selected
				if (Sketchup.read_default("Export All Textures","UMU",false) || Sketchup.read_default("Export All Textures","UNM",false))
					used_textures.each_key { |filename|
						entries_hash.delete(filename)
					}
					count = entries_hash.length
					entries_hash.each { |filename, filebase|
						#correct filename to alphanumeric characters
						filename = filename.delete('^A-Za-z0-9') if(Sketchup.is_valid_filename?(filename))
						#check if material exists with filename
						#and then create unique material name from filename
						i=0
						not_unique = true
						while not_unique
							i+=1
							matname = filename + ((i == 1) ? '' : i.to_s)
							found_matname = false
							Sketchup.active_model.materials.each do |mat|
								if(mat.display_name == matname)
									found_matname = true
									break
								end #if
							end #each
							not_unique = false if (!found_matname)
						end #while
						tick +=1
						msg_status="Creating Texture "+tick.to_s+" of "+count.to_s
						Sketchup::set_status_text(msg_status,SB_PROMPT)
						puts msg_status+": "+filebase
						#add a new material to the palette
						mat = Sketchup.active_model.materials.add(matname)
						mat.texture = fdirname + "/" + filebase
						mat.texture.size = [mat.texture.image_height,mat.texture.image_width]
						updated_materials[mat.name] = mat.name
					}
				end #if

				#consolidate texture files to texture folder on import if selected
				if (Sketchup.read_default("Export All Textures","CTF",true))
				  grp = Sketchup.active_model.entities.add_group
					tw = Sketchup.create_texture_writer
					fdirname = File.dirname(Sketchup.active_model.path) + "/textures"
					Dir.mkdir(fdirname, 755) if(!File.directory?(fdirname))
					
					tick = 0
					check = 0
					count = updated_materials.length
					updated_materials.each { |key, matname|
						tick+=1
						mat = Sketchup.active_model.materials[matname]
						msg_status="Consolidating Texture File "+tick.to_s+" of "+count.to_s
						Sketchup::set_status_text(msg_status,SB_PROMPT)
						puts msg_status+": "+mat.name
						
						next if (File.dirname(mat.texture.filename) == fdirname)
						grp.material = mat
						tw.load(grp)
						# should write texture here????
						File.delete(fdirname+"/"+File.basename(mat.texture.filename)) if(File.exists?(fdirname+"/"+File.basename(mat.texture.filename)))
						result = tw.write(grp, fdirname+"/"+File.basename(mat.texture.filename))
					
						case result
							when 0
								#puts "file written: "+fdirname+"/"+File.basename(mat.texture.filename)
								check+=1
							when 1; puts "file failed (invalid tiff): "+fdirname+"/"+File.basename(mat.texture.filename)
							when 2; puts "file failed (unknown): "+fdirname+"/"+File.basename(mat.texture.filename)
						end #case
						
						#update material texture filename
						tex_size = [mat.texture.width,mat.texture.height]		
						tex_alpha = mat.alpha
						tex_color = mat.color
						mat.texture = fdirname+"/"+File.basename(mat.texture.filename)
						mat.texture.size = tex_size
						mat.alpha = tex_alpha
						mat.color = tex_color
						
					}
					# Delete virtual face
					grp.erase! if grp.valid?
				end #if
				finish_time = Time.now-start
				msg_status="Operation Completed: Imported and Matched "+tick.to_s+" of "+count.to_s+" textures ("+(finish_time.to_s)+"s)"
				Sketchup::set_status_text(msg_status,SB_PROMPT)
				puts msg_status
				model.commit_operation
			else
				msg_status="Operation Failed: No valid file formats found. Run operation again with ruby console open to see error report."
				UI.messagebox msg_status, MB_OK	
				puts msg_status
				model.abort_operation
				Sketchup::set_status_text(msg_status,SB_PROMPT)
			end #if
			
		else
			msg_status="Operation Failed: Exported "+check.to_s+" of "+count.to_s+" textures. Run Operation again with ruby console open to see error report."
			puts msg_status
			Sketchup::set_status_text(msg_status,SB_PROMPT)
			model.abort_operation
		end #if
		
		
	end #def
	
	### get_entitys_texture added by bentleykf(niall.campbell@gmail.com)
	# gets entity's front and back texture and stores it in an
	# associative hash linking the material to the entity.
	# returns true if any texture found on entity(e), false otherwise
	def self.get_entitys_texture(e,v)
		if (e.respond_to?( :back_material ) || e.respond_to?( :material ))
			if e.respond_to?( :back_material )
				matb = e.back_material
			end #if
			if e.respond_to?( :material )
				mat = e.material
			end #if
		else
			return false
		end #if
		
		if (mat && mat.texture) || (matb && matb.texture)
			if v
				#kill the check when first material verified
				return true
			else
				#store the materials association to the entity
				if (mat && mat.texture)
					if(!@@material_assoc[mat.name])
						@@material_assoc[mat.name] = Array.new()
					end #if
					if(!@@material_assoc[mat.name][0])
						@@material_assoc[mat.name][0] = Array.new()
					end #if
					@@material_assoc[mat.name][0].push(e)
				end #if
				if (matb && matb.texture)
					if(!@@material_assoc[matb.name])
						@@material_assoc[matb.name] = Array.new()
					end #if
					if(!@@material_assoc[matb.name][1])
						@@material_assoc[matb.name][1] = Array.new()
					end #if
					@@material_assoc[matb.name][1].push(e)
				end #if
				return true
			end #if
		else
			return false
		end #if
		
	end #def
		
	### remove_entitys_texture added by bentleykf(niall.campbell@gmail.com)
	# removes entity's front and back texture from the
	# associative hash (@@material_assoc).
	# returns true if any texture found on entity(e), false otherwise
	def self.remove_entitys_texture(e)
		if (e.respond_to?( :back_material ) || e.respond_to?( :material ))
			if e.respond_to?( :back_material )
				matb = e.back_material
			end #if
			if e.respond_to?( :material )
				mat = e.material
			end #if
		else
			return false
		end #if
		
		if (mat && mat.texture) || (matb && matb.texture)
				#remove the materials association to the entity
				if (mat && mat.texture)
					if(@@material_assoc[mat.name] && @@material_assoc[mat.name][0])
						@@material_assoc[mat.name][0].delete(e)
					end #if
				end #if
				if (matb && matb.texture)
					if(@@material_assoc[matb.name] && @@material_assoc[matb.name][1])
						@@material_assoc[matb.name][1].delete(e)
					end #if
				end #if
				return true
		else
			return false
		end #if

	end #def
	
	### update_assoc added by bentleykf(niall.campbell@gmail.com)
	# a pass filter that corrects the components entities after
	# component has been made unique. Updates the @@material_assoc
	# hash by removing entities inside the old ComponentDefinition
	# (definition) and adding entities inside the new Component
	# Definition(component)
	def self.update_assoc(component,definition)
		has_groups = true
		has_textures = false
		ents = definition.entities.to_a
		new_ents = component.entities.to_a
		
		tick = 0
		ticks = 0
		while has_groups
			tick+=1
			puts "Pass "+tick.to_s+": "+ents.length.to_s+" entities to remove."
			
			if !ents[0]
				has_groups = false
				puts "Error: Nothing in entities collection. Can't update entities."
				return false
			end #if
			
			has_groups = false
			
			nents = ents.dup
			
			nents.each{|e|
				if(e.kind_of?(Sketchup::Group) || e.kind_of?(Sketchup::Face) || e.kind_of?(Sketchup::ComponentInstance) || e.kind_of?(Sketchup::ComponentDefinition) || e.kind_of?(Sketchup::Drawingelement))
					#check entity for material inheritance
					if (FastTW::FastTextureWriter.remove_entitys_texture(e))
						ticks+=1
						has_textures = true
					end #if
					#check entitys parent (DrawingElement) for material inheriatance
					if (FastTW::FastTextureWriter.remove_entitys_texture(e.parent))
						ticks+=1
						has_textures = true
					end #if
					
					if (e.kind_of?(Sketchup::Group))
						has_groups = true
						#merge entities of the group with the ents hash
						e.entities.each{|ent|
							ents.push(ent)
						}
					end #if
					
					#remove the entity from the ents array so each entity is only passed once
					ents.delete(e)
				end #if
			}
			if !has_groups
				if !has_textures
					puts "Error: No textures found in component. No textures to export/import."
					return false
				end
				if ents.length > 0
					has_groups = true
				end #if
			end #if
		end #while
		#puts "Remove complete: "+ticks.to_s+" entities removed"
		
		has_groups = true
		has_textures = false
		tick = 0
		ticks= 0
		while has_groups
			tick+=1
			puts "Pass "+tick.to_s+": "+new_ents.length.to_s+" entities to add."
			
			if !new_ents[0]
				has_groups = false
				puts "Error: Nothing in entities collection. Can't update entities."
				return false
			end #if
			
			has_groups = false
			
			nnew_ents = new_ents.dup
			
			nnew_ents.each{|e|
				if(e.kind_of?(Sketchup::Group) || e.kind_of?(Sketchup::Face) || e.kind_of?(Sketchup::ComponentInstance) || e.kind_of?(Sketchup::ComponentDefinition) || e.kind_of?(Sketchup::Drawingelement))
					#check entity for material inheritance
					if (FastTW::FastTextureWriter.get_entitys_texture(e,false))
						ticks+=1
						has_textures = true
					end #if
					#check entitys parent (DrawingElement) for material inheriatance
					if (FastTW::FastTextureWriter.get_entitys_texture(e.parent,false))
						ticks+=1
						has_textures = true
					end #if
					
					if (e.kind_of?(Sketchup::Group))
						has_groups = true
						#merge entities of the group with the new_ents hash
						e.entities.each{|ent|
							new_ents.push(ent)
						}
					end #if
					
					#remove the entity from the new_ents array so each entity is only passed once
					new_ents.delete(e)
				end #if
				if new_ents.length > 0
					has_groups = true
				end #if
			}
			#puts "Add complete: "+ticks.to_s+" entities added"
			if has_textures
				return true
			else
				puts "Error: No textures found in component. No textures to export/import."
				return false
				break
			end
		end #while
		
	end #def
	
	### sieve_entities added by bentleykf(niall.campbell@gmail.com)
	# a pass filter of all the entities. sieves through groups, components
	# and faces grabbing each unique association between an entity and
	# a texture. Stores each component instance in selection for sorting later.
	# Returns true if textures are found in entities hash(ents), false otherwise
	# Running in validation(v=true) ends the sieve when the first texture is found
	def self.sieve_entities(ents,v,msg)
		has_groups = true
		has_textures = false
		@@checked_components = Hash.new()
		
		tick = 0
		while has_groups
			tick+=1
			msg_status=msg+": Seiving Entities: Pass: "+tick.to_s+" ["+ents.length.to_s+"]"
			#puts msg_status
			Sketchup::set_status_text(msg_status,SB_PROMPT)	
			if !ents[0]
				has_groups = false
				#puts "Error: Nothing in entities collection. Can't run entity sieve."
				return false
				break
			end #if
			#switch has_groups to run following test for groups/components
			has_groups = false
			#test for groups/components and create a flattened array of entities
			dents = ents.dup
			dents.each{|e|
				if (e.kind_of?(Sketchup::Group) || e.kind_of?(Sketchup::ComponentInstance) || e.kind_of?(Sketchup::Face) || e.kind_of?(Sketchup::ComponentDefinition) || e.kind_of?(Sketchup::Drawingelement)) 
					#check entity for material inheritance
					if (FastTW::FastTextureWriter.get_entitys_texture(e,v))
						return true if v
						has_textures = true
					end #if
					#check entitys parent (DrawingElement) for material inheriatance
					if (FastTW::FastTextureWriter.get_entitys_texture(e.parent,v))
						return true if v
						has_textures = true
					end #if
				end #if
			
				if (e.kind_of?(Sketchup::Group))
					has_groups = true
					#merge entities of the group with the ents hash
					e.entities.each{|ent|
						ents.push(ent)
					}
				end #if
				if (e.kind_of?(Sketchup::ComponentInstance))
					has_groups = true
					#if not already passed, merge the components entities
					#to the ents hash
					if !@@checked_components[e.definition.guid]
						@@checked_components[e.definition.guid] = [e]
						e.definition.entities.each{|ent|
							ents.push(ent)
						}
					else
						@@checked_components[e.definition.guid] = @@checked_components[e.definition.guid] | [e]
					end #if
				end #if
				
				#remove the entity from the ents array so each entity is only passed once
				ents.delete(e)
			}
			if !has_groups
				if has_textures
					has_groups = false
					return true
				else
					puts "Error: No textures found in collection. No textures to export/import."
					return false
				end
			end #if
		end #while
		
		#return a false value for validation
		return false
	end #def
	
	### fast_tws added by bentleykf(niall.campbell@gmail.com)
	# Grabs all the valid textures in selection then attempts to
	# export them using TextureWriter.
	def self.fast_tws(ents) 
		@@material_assoc = Hash.new()
		@@checked_components = Hash.new()
	
    model=Sketchup.active_model
		#run Jim's sketchup version check
		if Sketchup.version.to_f < 7.0
			model.start_operation("Export From Selected",true)
		else
			model.start_operation("Export From Selected")
		end #if
		
		start = Time.now
		#first sieve out the groups and components
		if(FastTW::FastTextureWriter.sieve_entities(ents,false,"Export From Selection"))
			#make textures folder if it doesn't currently exist
			fdirname = File.dirname(Sketchup.active_model.path) + "/textures/"
			Dir.mkdir(fdirname, 755) if(!File.directory?(fdirname))

			selected_material_assoc = Hash.new()
			#1.1   First run through all materials and give them unique names (FastTW::FastTextureWriter.correct_filenames)
			FastTW::FastTextureWriter.correct_filenames("Export From Selection")
			
			
			#3.    If Painted Components Unique(SCU) now we need to loop through components and make them unique
			if Sketchup.read_default("Export All Textures","SCU",true)
				if(@@checked_components.length > 0)
					tick=0
					count = @@checked_components.length
					@@checked_components.each {|guid,instances|
						component = Sketchup.active_model.definitions[guid]
						#3.1  Exception: components contained within selection, do nothing
						if(instances.length < component.instances.length)
							definition = false
							tick+=1
							msg_status="Export From Selection: Updating "+tick.to_s+" of "+count.to_s+" Non Unique Components"
							puts msg_status
							Sketchup::set_status_text(msg_status,SB_PROMPT)		
							instances.each{|instance|
								#make unique definition for components in selection
								if (definition == false)
									instance.make_unique
									definition = instance.definition
									#3.2   May need to update entity ids for each component????
									#remove old entities from material_assoc array and add these entities?
									#FastTW::FastTextureWriter.update_assoc(component,definition)
									FastTW::FastTextureWriter.update_assoc(definition,component)
								end #if
								#update identical components to the unique definition
								instance.definition = definition
							}
						end #if
					}
				end #if
			end #if
			
			#1.2   Loop through the material assoc hash
			selected_material_assoc = @@material_assoc.dup
			#selected_material_assoc.merge(@@material_assoc.dup) #copy the material assoc hash
			#selected_material_assoc.replace(@@material_assoc)  #copy the material assoc hash
			@@material_assoc = Hash.new()												#reset the material_assoc hash		
			# will only need to do the following if materials unique to selection selected
			if Sketchup.read_default("Export All Textures","MUS",true)	
				FastTW::FastTextureWriter.sieve_entities(Sketchup.active_model.entities.to_a,false,"Export From Selection")	#fill material_assoc hash with all models textures
			end #if
			
			#2.    Export for each unique texture:
			# Initialise the texturewriter
			checked_filenames = Hash.new()
			grp = Sketchup.active_model.entities.add_group
			tw = Sketchup.create_texture_writer
			
			tick = 0
			check = 0
			count = selected_material_assoc.length
			
			dup_selected_material_assoc = selected_material_assoc.dup
			
			dup_selected_material_assoc.each {|mat_name,mat_assoc_array|
				#2.1   If materials unique to selection(MUS) we need to create unique names for all textures
				mat = Sketchup.active_model.materials[mat_name]	
				tex_size = [mat.texture.width,mat.texture.height]
				tex_alpha = mat.alpha
				tex_color = mat.color
				filebase = File.basename(mat.texture.filename)
				fileext  = File.extname(mat.texture.filename)
				filename = filebase[0,filebase.length-fileext.length]
				filename = filename.delete('^A-Za-z0-9_')
				if Sketchup.read_default("Export All Textures","MUS",true)
					#puts "material unique to selection thrown"
					#2.1.1 Exception: texture is contained within selection, update existing material
					# check if material isn't unique to the selection
					
					if (mat_assoc_array && mat_assoc_array[0] && @@material_assoc[mat_name] && @@material_assoc[mat_name][0] && (mat_assoc_array[0].length < @@material_assoc[mat_name][0].length)) || (mat_assoc_array && mat_assoc_array[1] && @@material_assoc[mat_name] && @@material_assoc[mat_name][1] && (mat_assoc_array[1].length < @@material_assoc[mat_name][1].length))
						#material isn't unique to the selection, make material unique
						#puts "material not unique to selection:"+mat_name.to_s
						#create unique filename
						i=1
						if !checked_filenames[filename+(i.to_s)]
							checked_filenames[filename+(i.to_s)] = filebase
							filename = filename+(i.to_s)
						else
							#now make this filename unique
							if(checked_filenames[filename])
								not_unique = true
								while not_unique
									i+=1
									if(!checked_filenames[filename+(i.to_s)])
										filename = filename + (i.to_s)
										not_unique = false
									end #if
								end #while
							end #if
							
							checked_filenames[filename] = filebase						
						end #if
						
						#now we have a unique filename, we can create a new material with the filename property and then update its texture on export
						
						#add a new material to the palette
						#puts "material made unique to selection"
						mat2 = Sketchup.active_model.materials.add(filename)
						new_name = mat2.name
						#puts "material name: "+filename
						#might need to correct selected_material_assoc hash here???
						#2.1.1.1 Store updated material_assoc data
						selected_material_assoc[new_name] = Array.new()
						selected_material_assoc[new_name] = selected_material_assoc[mat_name].dup
						selected_material_assoc.delete(mat_name)
					else
						mat2 = mat
					end #if
				else
					mat2 = mat
				end #if
				tick +=1
				msg_status="Export From Selection: Writing Texture "+tick.to_s+" of "+count.to_s
				Sketchup::set_status_text(msg_status,SB_PROMPT)		
				
				#2.1.2 Export all materials using texturewriter
				grp.material = mat
				tw.load(grp)
				filenbase = fdirname+filename+fileext
				result = tw.write(grp, filenbase)

				case result
					when 0
						puts "file written: "+fdirname+filename+fileext
						check+=1
						mat2.texture = filenbase
						mat2.alpha = tex_alpha
						mat2.color = tex_color
						mat2.texture.size = tex_size
					when 1; puts "file failed (invalid tiff): "+filenbase
					when 2; puts "file failed (unknown): "+filenbase
				end #case
				
			}
			
			grp.erase! if grp.valid?

			
			#4.    Update all faces, groups and components to the new materials
			tick=0
			count = selected_material_assoc.length
			selected_material_assoc.each {|mat_name,mat_assoc_array|
				tick+=1
				mat = Sketchup.active_model.materials[mat_name]
				if mat_assoc_array[0]
					ticks=0
					counts=mat_assoc_array[0].length
					mat_assoc_array[0].each {|e|
						ticks+=1
						msg_status="Export From Selection: Updating Front Material "+tick.to_s+" of "+count.to_s+" ("+mat_name.to_s+"); Entity:"+ticks.to_s+" of "+counts.to_s
						Sketchup::set_status_text(msg_status,SB_PROMPT)		
						e.material = mat
					}
				end #if
				if mat_assoc_array[1]
					ticks=0
					counts=mat_assoc_array[1].length
					mat_assoc_array[1].each {|e|
						ticks+=1
						msg_status="Export From Selection: Updating Back Material "+tick.to_s+" of "+count.to_s+" ("+mat_name.to_s+"); Entity:"+ticks.to_s+" of "+counts.to_s
						Sketchup::set_status_text(msg_status,SB_PROMPT)
						e.back_material = mat
					}
				end #if
			}
			
			if (check == count)
				finish_time = Time.now-start
				msg_status="Operation Completed: Exported "+check.to_s+" of "+count.to_s+" textures ("+(finish_time.to_s)+"s)"
				puts msg_status
				Sketchup::set_status_text(msg_status,SB_PROMPT)		
				model.commit_operation
			else
				msg_status="Operation Failed: Exported "+check.to_s+" of "+count.to_s+" textures. Run Operation again with ruby console open to see error report."
				UI.messagebox msg_status, MB_OK
				Sketchup::set_status_text(msg_status,SB_PROMPT)	
				puts msg_status
				model.abort_operation
			end #if
			
		end #if
	end #def
	
	### fast_tws added by bentleykf(niall.campbell@gmail.com)
	# Grabs all the valid textures in folder then compares with
	# textures in selection and imports successful namespace
	# matches
	def self.fast_trs(ents)
		@@material_assoc = Hash.new()
		@@checked_components = Hash.new()
		#1. Open files and check validity
    model=Sketchup.active_model
		#run Jim's sketchup version check
		if Sketchup.version.to_f < 7.0
			model.start_operation("Import To Selected",true)
		else
			model.start_operation("Import To Selected")
		end #if
		
		start = Time.now
		dirpath = UI.openpanel("Choose A File Inside The Texture Import Folder", File.dirname(Sketchup.active_model.path), "*.*")
		if (dirpath)
			fdirname = File.dirname(dirpath)
			
			entries_hash = FastTW::FastTextureWriter.check_validity(fdirname)
			if (entries_hash && entries_hash.length > 0)
				#2. Get selection and check for materials
				if(FastTW::FastTextureWriter.sieve_entities(ents,false,"Import From Selection"))
					#3. Check for namespace equivalents between materials and files (and assoc matname to file)

					update_materials = Hash.new()
					tick = 0
					count = @@material_assoc.length
					@@material_assoc.each{|mat_name,mat_assoc_array|
						tick+=1
						mat = Sketchup.active_model.materials[mat_name]
						filebase = File.basename(mat.texture.filename)
						fileext  = File.extname(mat.texture.filename)
						filename = filebase[0,filebase.length-fileext.length]
						msg_status="Import From Selection: Checking namespace similarity "+tick.to_s+" of "+count.to_s+""
						if(entries_hash[filename])
							update_materials[mat_name] = fdirname+"/"+entries_hash[filename]
							msg_status="Import From Selection: Found namespace similarity "+tick.to_s+" of "+count.to_s+":"+fdirname+"/"+entries_hash[filename]
							puts msg_status
						end #if
					}
					
					if(update_materials.length > 0)
						
						tick = 0
						check = 0
						#4.1.1 Check for component uniqueness
						if Sketchup.read_default("Export All Textures","SCU",true)
							if(@@checked_components.length > 0)
								tick=0
								count = @@checked_components.length
								@@checked_components.each {|guid,instances|
									component = Sketchup.active_model.definitions[guid]
									#4.1  Exception: components contained within selection, do nothing
									if(instances.length < component.instances.length)
										definition = false
										tick+=1
										msg_status="Import From Selection: Updating "+tick.to_s+" of "+count.to_s+" Non Unique Components"
										puts msg_status
										Sketchup::set_status_text(msg_status,SB_PROMPT)		
										instances.each{|instance|
											#make unique definition for components in selection
											if (definition == false)
												instance.make_unique
												definition = instance.definition
												#puts "component made unique"
												#4.2   May need to update entity ids for each component????
												#remove old entities from material_assoc array and add these entities?
												#FastTW::FastTextureWriter.update_assoc(component,definition)
												FastTW::FastTextureWriter.update_assoc(definition,component)
												
											end #if
											#update identical components to the unique definition
											instance.definition = definition
										}
									end #if
								}
							end #if
						end #if
						
						selected_material_assoc = @@material_assoc.dup
						#selected_material_assoc.merge(@@material_assoc.dup) #copy the material assoc hash
						#selected_material_assoc.replace(@@material_assoc)  #copy the material assoc hash
						@@material_assoc = Hash.new()												#reset the material_assoc hash		
						
						#5. Check for material uniqueness and modify matname if not unique
						if Sketchup.read_default("Export All Textures","MUS",true)	
							FastTW::FastTextureWriter.sieve_entities(Sketchup.active_model.entities.to_a,false,"Import From Selection")	#fill material_assoc hash with all models textures
						end #if
						
						# Create virtual face
						grp = Sketchup.active_model.entities.add_group
						tw = Sketchup.create_texture_writer
						
						count = update_materials.length
						
						new_material_assoc = Hash.new()
						
						checked_filenames = Hash.new()
						
						dup_update_materials = update_materials.dup
						
						tick = 0
						check = 0	
						
						dup_update_materials.each {|mat_name,new_filename|
							#2.1   If materials unique to selection(MUS) we need to create unique names for all textures
							mat = Sketchup.active_model.materials[mat_name]	
							tex_size = [mat.texture.width,mat.texture.height]
							tex_alpha = mat.alpha
							tex_color = mat.color
							filebase = File.basename(mat.texture.filename)
							fileext  = File.extname(mat.texture.filename)
							filename = filebase[0,filebase.length-fileext.length]
							filename = filename.delete('^A-Za-z0-9_')
							is_unique = true
							if Sketchup.read_default("Export All Textures","MUS",true)
								#puts "material unique to selection thrown"
								#2.1.1 Exception: texture is contained within selection, update existing material
								# check if material isn't unique to the selection
								mat_assoc_array = selected_material_assoc[mat_name]
								if (mat_assoc_array && mat_assoc_array[0] && @@material_assoc[mat_name] && @@material_assoc[mat_name][0] && (mat_assoc_array[0].length < @@material_assoc[mat_name][0].length)) || (mat_assoc_array && mat_assoc_array[1] && @@material_assoc[mat_name] && @@material_assoc[mat_name][1] && (mat_assoc_array[1].length < @@material_assoc[mat_name][1].length))
									#material isn't unique to the selection, make material unique
									#puts "material not unique to selection:"+mat_name.to_s
									#create unique filename
									i=1
									if !checked_filenames[filename+(i.to_s)] && !entries_hash[filename+(i.to_s)]
										checked_filenames[filename+(i.to_s)] = filebase
										filename = filename+(i.to_s)
									else
										#now make this filename unique
										if(checked_filenames[filename+(i.to_s)] || entries_hash[filename+(i.to_s)])
											not_unique = true
											while not_unique
												i+=1
												if(!checked_filenames[filename+(i.to_s)] && !entries_hash[filename+(i.to_s)])
													filename = filename + (i.to_s)
													not_unique = false
												end #if
											end #while
										end #if
										
										checked_filenames[filename] = filebase						
									end #if
									
									#now we have a unique filename, we can create a new material with the filename property and then update its texture on export
									
									#add a new material to the palette
									#puts "material made unique to selection"
									mat2 = Sketchup.active_model.materials.add(filename)
									update_materials[filename] = new_filename
									update_materials.delete(mat_name)
									#puts "material name: "+filename
									#might need to correct selected_material_assoc hash here???
									#2.1.1.1 Store updated material_assoc data
									new_material_assoc[filename] = Array.new()
									new_material_assoc[filename] = selected_material_assoc[mat_name]
									mat_name = filename
									puts "creating new material:"+mat_name.to_s
									is_unique = false
								else
									mat2 = mat
									new_material_assoc[mat_name] = Array.new()
									new_material_assoc[mat_name] = selected_material_assoc[mat_name]
								end #if
							else
								mat2 = mat
								new_material_assoc[mat_name] = Array.new()
								new_material_assoc[mat_name] = selected_material_assoc[mat_name]
							end #if
							tick +=1
							msg_status="Import From Selection: Importing Texture "+tick.to_s+" of "+count.to_s
							puts msg_status
							Sketchup::set_status_text(msg_status,SB_PROMPT)
							#6. Update material with new texture
							mat2.texture = new_filename
							mat2.alpha = tex_alpha
							mat2.color = tex_color
							mat2.texture.size = tex_size
							if (!is_unique)
								#try to resolve potential namespace conflicts between the models
								#texture and this new texture
								grp.material = mat2
								tw.load(grp)
								# should write texture here????
								unique_filename = File.dirname(new_filename)+"/"+mat_name+fileext
								result = tw.write(grp, unique_filename)
							
								case result
									when 0
										puts "file written: "+unique_filename
										mat2.texture = unique_filename
										update_materials[mat_name] = unique_filename
										mat2.texture.size = tex_size
										mat2.alpha = tex_alpha
										mat2.color = tex_color
									when 1; puts "file failed (invalid tiff): "+unique_filename
									when 2; puts "file failed (unknown): "+unique_filename
								end #case
							end #if
						}
						
						#5.1 Update entitys in model with new matname
						#if Sketchup.read_default("Export All Textures","MUS",true)	
							tick=0
							count = new_material_assoc.length
							new_material_assoc.each {|mat_name,mat_assoc_array|
								tick+=1
								if (mat_name)
									mat = Sketchup.active_model.materials[mat_name]
									if (mat_assoc_array)
										if mat_assoc_array[0]
											ticks=0
											counts=mat_assoc_array[0].length
											mat_assoc_array[0].each {|e|
												ticks+=1
												msg_status="Import From Selection: Updating Front Material "+tick.to_s+" of "+count.to_s+"; Entity:"+ticks.to_s+" of "+counts.to_s
												Sketchup::set_status_text(msg_status,SB_PROMPT)	
												e.material = mat
											}
										end #if
										if mat_assoc_array[1]
											ticks=0
											counts=mat_assoc_array[1].length
											mat_assoc_array[1].each {|e|
												ticks+=1
												msg_status="Import From Selection: Updating Back Material "+tick.to_s+" of "+count.to_s+"; Entity:"+ticks.to_s+" of "+counts.to_s
												Sketchup::set_status_text(msg_status,SB_PROMPT)		
												e.back_material = mat
											}
										end #if
									else
										msg_status="Import From Selection: No Associated Entities:"+mat_name.to_s
										puts msg_status
									end
								else
									msg_status="Import From Selection: Material Name Invalid:"+mat_name.to_s
									puts msg_status
								end #if
							}
						#end #if
						
						#7. Consolidate texture library if needed
						
						if Sketchup.read_default("Export All Textures","SCT",true)
							fdirname = File.dirname(Sketchup.active_model.path) + "/textures"
							Dir.mkdir(fdirname, 755) if(!File.directory?(fdirname))
							
							tick = 0
							count = update_materials.length
							check = 0
							update_materials.each { |matname, new_filename|
								tick+=1
								mat = Sketchup.active_model.materials[matname]
								msg_status="Import From Selection: Consolidating Texture File "+tick.to_s+" of "+count.to_s
								Sketchup::set_status_text(msg_status,SB_PROMPT)
								#puts msg_status+": "+mat.name
								
								next if (File.dirname(mat.texture.filename) == fdirname)
								grp.material = mat
								tw.load(grp)
								# should write texture here????
								#File.delete(fdirname+"/"+File.basename(mat.texture.filename)) if(File.exists?(fdirname+"/"+File.basename(mat.texture.filename)))
								result = tw.write(grp, fdirname+"/"+File.basename(mat.texture.filename))
							
								case result
									when 0
										puts "file written: "+fdirname+"/"+File.basename(mat.texture.filename)
										check+=1
									when 1; puts "file failed (invalid tiff): "+fdirname+"/"+File.basename(mat.texture.filename)
									when 2; puts "file failed (unknown): "+fdirname+"/"+File.basename(mat.texture.filename)
								end #case
								
								#update material texture filename
								tex_size = [mat.texture.width,mat.texture.height]
								tex_alpha = mat.alpha
								tex_color = mat.color								
								mat.texture = fdirname+"/"+File.basename(mat.texture.filename)
								mat.texture.size = tex_size
								mat.alpha = tex_alpha
								mat.color = tex_color
								
							}
							
							if (check == count)
								finish_time = Time.now-start
								msg_status="Operation Completed: Imported "+check.to_s+" of "+count.to_s+" textures to selection ("+(finish_time.to_s)+"s)"
								puts msg_status
								Sketchup::set_status_text(msg_status,SB_PROMPT)		
								model.commit_operation
								# Delete virtual face
								grp.erase! if grp.valid?
								return true
							else
								msg_status="Operation Failed: Imported "+check.to_s+" of "+count.to_s+" textures to selection. Run Operation again with ruby console open to see error report."
								UI.messagebox msg_status, MB_OK
								Sketchup::set_status_text(msg_status,SB_PROMPT)	
								puts msg_status
								model.abort_operation
								# Delete virtual face
								grp.erase! if grp.valid?
								return false
							end #if
						end #if
						# Delete virtual face
						grp.erase! if grp.valid?
						finish_time = Time.now-start
						msg_status="Operation Completed: Imported "+count.to_s+" of "+count.to_s+" textures to selection ("+(finish_time.to_s)+"s)"
						puts msg_status
						Sketchup::set_status_text(msg_status,SB_PROMPT)
						model.commit_operation
					else	
						msg_status="Operation Failed: No match between files and textured materials in selection. Run operation again with ruby console open to see error report."
						UI.messagebox msg_status, MB_OK	
						puts msg_status
						model.abort_operation
						Sketchup::set_status_text(msg_status,SB_PROMPT)
					end #if

				else
					msg_status="Operation Failed: No textured materials in selection. Run operation again with ruby console open to see error report."
					UI.messagebox msg_status, MB_OK	
					puts msg_status
					model.abort_operation
					Sketchup::set_status_text(msg_status,SB_PROMPT)
				end #if
			else
				msg_status="Operation Failed: No valid file formats found. Run operation again with ruby console open to see error report."
				UI.messagebox msg_status, MB_OK	
				puts msg_status
				model.abort_operation
				Sketchup::set_status_text(msg_status,SB_PROMPT)
			end #if
		else
			msg_status="Operation Failed: No File Chosen."
			puts msg_status
			model.abort_operation
			Sketchup::set_status_text(msg_status,SB_PROMPT)
		end #if
	end #def
	
end #class
end #module

### menu added by takesh
if(not file_loaded?("fast_tw_3.rb"))

	# set defaults if none present
	Sketchup.write_default("Export All Textures","UTP",true) if Sketchup.read_default("Export All Textures","UTP","fail") == "fail"
	Sketchup.write_default("Export All Textures","UME",true) if Sketchup.read_default("Export All Textures","UME","fail") == "fail"
  Sketchup.write_default("Export All Textures","UMU",false) if Sketchup.read_default("Export All Textures","UMU","fail") == "fail"
	Sketchup.write_default("Export All Textures","UNM",false) if Sketchup.read_default("Export All Textures","UNM","fail") == "fail"
	Sketchup.write_default("Export All Textures","CTF",true) if Sketchup.read_default("Export All Textures","CTF","fail") == "fail"
	Sketchup.write_default("Export All Textures","SCU",false) if Sketchup.read_default("Export All Textures","SCU","fail") == "fail"
	Sketchup.write_default("Export All Textures","MUS",false) if Sketchup.read_default("Export All Textures","MUS","fail") == "fail"
	Sketchup.write_default("Export All Textures","SCT",true) if Sketchup.read_default("Export All Textures","SCT","fail") == "fail"
	
	### submenu added by bentleykf(niall.campbell@gmail.com)
	submenu = UI.menu("Plugins").add_submenu("Export All Textures")
	cmd = UI::Command.new("Export All Textures") {
		FastTW::FastTextureWriter.fast_tw
	}
	
	#check if model has textures to export
	cmd.set_validation_proc {
		has_textures = false
		if (Sketchup.active_model.materials.length >= 1)
			Sketchup.active_model.materials.each do |mat|
				if(mat.texture) 
					has_textures = true
					break #break each loop
				end #if
			end #each
		end #if
		(has_textures) ? MF_ENABLED : MF_GRAYED
	}
	submenu.add_item(cmd)
	
	cmd = UI::Command.new("Export From Selection") {
		FastTW::FastTextureWriter.fast_tws(Sketchup.active_model.selection.to_a)
	}
	
	#check if selection has textures to export
	cmd.set_validation_proc {
		has_textures = FastTW::FastTextureWriter.sieve_entities(Sketchup.active_model.selection.to_a,true,'')
		(has_textures) ? MF_ENABLED : MF_GRAYED
	}
	
	submenu.add_item(cmd)
	
	submenu.add_item("Import All Textures") {FastTW::FastTextureWriter.fast_tr}
	
	cmd = UI::Command.new("Import To Selection") {
		FastTW::FastTextureWriter.fast_trs(Sketchup.active_model.selection.to_a)
	}
	
	#check if selection has textures to export
	cmd.set_validation_proc {
		has_textures = FastTW::FastTextureWriter.sieve_entities(Sketchup.active_model.selection.to_a,true,'')
		(has_textures) ? MF_ENABLED : MF_GRAYED
	}
	
	submenu.add_item(cmd)
	
	submenu.add_separator
	
	settings_menu = submenu.add_submenu("Export Settings")
	
	cmd = UI::Command.new("Update Model's Texture Paths") {
		update_paths = Sketchup.read_default("Export All Textures","UTP",true)
		if update_paths
			Sketchup.write_default("Export All Textures","UTP",false)
		else
			Sketchup.write_default("Export All Textures","UTP",true)
		end
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","UTP",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	settings_menu = submenu.add_submenu("Import Settings")

	# Only Update Existing Materials (UME)
	cmd = UI::Command.new("Only Update Existing Materials") {
		Sketchup.write_default("Export All Textures","UME",true)
		Sketchup.write_default("Export All Textures","UMU",false)
		Sketchup.write_default("Export All Textures","UNM",false)
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","UME",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	# Create New Materials for Unique Textures (UMU)
	cmd = UI::Command.new("Create New Materials for Unique Textures") {
		Sketchup.write_default("Export All Textures","UME",false)
		Sketchup.write_default("Export All Textures","UMU",true)
		Sketchup.write_default("Export All Textures","UNM",false)
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","UMU",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	# Create new Materials for all textures (UNM)
	cmd = UI::Command.new("Create New Materials for all Textures") {
		Sketchup.write_default("Export All Textures","UME",false)
		Sketchup.write_default("Export All Textures","UMU",false)
		Sketchup.write_default("Export All Textures","UNM",true)
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","UNM",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	settings_menu.add_separator
	
	# Create new Materials for all textures (UNM)
	cmd = UI::Command.new("Consolidate to Texture Folder") {
		consolidate = Sketchup.read_default("Export All Textures","CTF",true)
		if consolidate
			Sketchup.write_default("Export All Textures","CTF",false)
		else
			Sketchup.write_default("Export All Textures","CTF",true)
		end
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","CTF",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	settings_menu = submenu.add_submenu("Selection Settings")

	# Make Painted Components Unique (SCU)
	cmd = UI::Command.new("Make Painted Components Unique") {
		update_paths = Sketchup.read_default("Export All Textures","SCU",true)
		if update_paths
			Sketchup.write_default("Export All Textures","SCU",false)
		else
			Sketchup.write_default("Export All Textures","SCU",true)
		end
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","SCU",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	# Make Materials Unique to Selection (SCU)
	cmd = UI::Command.new("Materials Unique to Selection") {
		update_paths = Sketchup.read_default("Export All Textures","MUS",true)
		if update_paths
			Sketchup.write_default("Export All Textures","MUS",false)
		else
			Sketchup.write_default("Export All Textures","MUS",true)
		end
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","MUS",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
	# Consolidate to Texture Folder on Import (SCT)
	cmd = UI::Command.new("Consolidate to Texture Folder on Import") {
		consolidate = Sketchup.read_default("Export All Textures","SCT",true)
		if consolidate
			Sketchup.write_default("Export All Textures","SCT",false)
		else
			Sketchup.write_default("Export All Textures","SCT",true)
		end
	}
	
	cmd.set_validation_proc {Sketchup.read_default("Export All Textures","SCT",true) ? MF_CHECKED : MF_UNCHECKED}
	settings_menu.add_item(cmd)
	
end

file_loaded("fast_tw_3.rb")
###